// infblock.c -- interpret and process block types to last block
///////////////////////////////////////////////////////////////////////////////////////////////////
#include "DevUnzip.h"
///////////////////////////////////////////////////////////////////////////////////////////////////
// TABLE FOR DEFLATE FROM PKZIP'S APPNOTE.TXT.  
const UInt ProgMemmory  _DefBorder[20]={
        16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15, 0};  

PUInt DefBorder = (PUInt) _DefBorder;
///////////////////////////////////////////////////////////////////////////////////////////////////
/*
   Notes beyond the 1.93a appnote.txt:

   1. Distance pointers never point before the beginning of the output
      stream.
   2. Distance pointers can point back across blocks, up to 32k away.
   3. There is an implied maximum of 7 bits for the bit length table and
      15 bits for the actual data.
   4. If only one code exists, then it is encoded using one bit.  (Zero
      would be more efficient, but perhaps a little confusing.)  If two
      codes exist, they are coded using one bit each (0 and 1).
   5. There is no way of sending zero distance codes--a dummy must be
      sent if there are none.  (History: a pre 2.0 version of PKZIP would
      store blocks with no distance codes, but this was discovered to be
      too harsh a criterion.)  Valid only for 1.93a.  2.04c does allow
      zero distance codes, which is sent as one code of zero bits in
      length.
   6. There are up to 286 literal/length codes.  Code 256 represents the
      end-of-block.  Note however that the static length tree defines
      288 codes just to fill out the Huffman codes.  Codes 286 and 287
      cannot be used though, since there is no length Base or extra bits
      defined for them.  Similarily, there are up to 30 distance codes.
      However, static trees define 32 codes (all 5 bits) to fill out the
      Huffman codes, but the LastBlk two had better not show up in the data.
   7. Unzip can check dynamic Huffman blocks for complete code sets.
      The exception is that a single code would not be complete (see #4).
   8. The five bits following the block type is really the number of
      literal codes sent minus 257.
   9. Length codes 8,16,16 are interpreted as 13 length codes of 8 bits
      (1+6+6).  Therefore, to output three times the length, you output
      three codes (1+1+1), whereas to output four times the same length,
      you only need two codes (1+3).  Hmm.
  10. In the tree reconstruction algorithm, Code = Code + Increment
      only if BitLength(i) is not zero.  (Pretty obvious.)
  11. Correction: 4 Bits: # of Bit Length codes - 4     (4 - 19)
  12. Note: length code 284 can represent 227-258, but length code 285
      really is 258.  The last length deserves its own, Short code
      since it gets used a lot in very redundant files.  The length
      258 is special since 258 - 3 (the min match length) is 255.
  13. The literal/length and distance code bit lengths are read as a
      single stream of lengths.  It is possible (and advantageous) for
      a repeat code (16, 17, or 18) to go across the boundary between
      the two sets of lengths.
 */
///////////////////////////////////////////////////////////////////////////////////////////////////
void InflateBlocksReset(void)
{
    if((IBState->InfBlockMode == BLK_BTREE) 
    || (IBState->InfBlockMode == BLK_DTREE)){
      if(IBState->SubMode.Trees.BitLens){
         UnZipFree(IBState->SubMode.Trees.BitLens);
      }
    }
    UnZipFree((PByte)-1);
    IBState->InfBlockMode      = BLK_TYPE;
    IBState->BitsLeft          = 0;
    IBState->BitBuffer         = 0;
    IBState->WinRead           = IBState->WinWrite = IBState->WinSlide;

    Zee->CheckSum = IBState->OutCheck = CheckSum32(0L, (PByte)Z_NULL, 0);
    Tracev((stderr, "inflate:   blocks reset\n"));
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void InflateBlocksNew         (void)
{
UInt WindowByteSize            = 0x8000;

    ZMemClear ((PByte)IBState,  SIZEOF_BLOCKSTATE);   

    IBState->WinSlide          = LZ77Window;
    IBState->hufts             = HuftmanTable;

    IBState->WinEnd            = 
    IBState->WinSlide          + WindowByteSize;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateBlocks(Int RetCode)
{
UInt    TempVal;                // TEMPORARY STORAGE 
ULong   BitBuffer;              // BIT BUFFER 
UInt    BitsInBitBuffer;        // BITS IN BIT BUFFER 
PByte   InputDataPtr;           // INPUT DATA POINTER 
UInt    InputAvalaible;         // BYTES AVAILABLE THERE 
PByte   OutputWindowPtr;        // OUTPUT WINDOW WRITE POINTER 
UInt    WinEndCount;            // BYTES TO END OF WINDOW OR READ POINTER 

    LOAD // COPY INPUT/OUTPUT INFORMATION TO LOCALS (UPDATE MACRO RESTORES) 
    
    while (1) switch (IBState->InfBlockMode)// PROCESS INPUT BASED ON CURRENT STATE 
    {
    case BLK_TYPE:
         NEEDBITS(3)
         TempVal          = (UInt)BitBuffer & 7;
         IBState->LastBlk = TempVal & 1;
         switch (TempVal >> 1) //  THIRD BYTE BIT 1
         {
         case 0: // BLK_STORED 
              Tracev((stderr, "inflate:     stored block%s\n", IBState->LastBlk ? " (LastBlk)" : ""));
              DUMPBITS(3)
              TempVal = BitsInBitBuffer & 7;                    // GO TO BYTE BOUNDARY 
              DUMPBITS(TempVal)
              IBState->InfBlockMode = BLK_LENS; // GET LENGTH OF BLK_STORED BLOCK 
              break;
         case 1: // FIXED 
              Tracev((stderr, "inflate:     fixed codes block%s\n", IBState->LastBlk ? " (LastBlk)" : ""));
              {
                UInt           LitBitDepth, DistBitDepth;
                TInflate_Huft *LitTree, *DistTree;

                InflateTreesFixed (&LitBitDepth, &DistBitDepth, &LitTree, &DistTree);
                InflateCodesReset (LitBitDepth, DistBitDepth, LitTree, DistTree);
              }
              DUMPBITS(3)
              IBState->InfBlockMode = BLK_CODES;
              break;
         case 2: // DYNAMIC 
              Tracev((stderr, "inflate:     dynamic codes block%s\n", IBState->LastBlk ? " (LastBlk)" : ""));
              DUMPBITS(3)
              IBState->InfBlockMode = BLK_TABLE;
              break;
         case 3: // ILLEGAL 
              DUMPBITS(3)
              IBState->InfBlockMode = BLK_BAD;
              ReportErrorStr("invalid block type", Z_BAD_BLOCKTYPE);
              RetCode               = Z_DATA_ERROR;
              LEAVE
         }
         break;
    case BLK_LENS:
         NEEDBITS(32)
         if((((~BitBuffer) >> 16) & 0xffff) != (BitBuffer & 0xffff))
         {
           IBState->InfBlockMode = BLK_BAD;
           ReportErrorStr("invalid stored block lengths", Z_BAD_BLOCKLEN);
           RetCode               = Z_DATA_ERROR;
           LEAVE
         }
         IBState->SubMode.BytesLeft  = (UInt)BitBuffer & 0xffff;
         BitBuffer = BitsInBitBuffer = 0;                      // dump bits 
         Tracev((stderr, "inflate:       stored length %u\n", IBState->SubMode.BytesLeft));
         IBState->InfBlockMode = IBState->SubMode.BytesLeft ? BLK_STORED : (IBState->LastBlk ? BLK_DRY : BLK_TYPE);
         break;
    case BLK_STORED:
         if(InputAvalaible == 0){
           LEAVE
         }
         NEEDOUT
         TempVal = IBState->SubMode.BytesLeft;
         if(TempVal > InputAvalaible) { TempVal = InputAvalaible; }
         if(TempVal > WinEndCount)    { TempVal = WinEndCount;    }
         ZMemCopy(OutputWindowPtr, InputDataPtr, TempVal);
         InputDataPtr += TempVal;  InputAvalaible -= TempVal;
         OutputWindowPtr += TempVal;  WinEndCount -= TempVal;
         if((IBState->SubMode.BytesLeft -= TempVal) != 0){
           break;
         }
         Tracev((stderr, "inflate:       stored end, %lu total out\n",
                Zee->Total_out + (OutputWindowPtr >= IBState->WinRead ? OutputWindowPtr - IBState->WinRead :
                (IBState->WinEnd - IBState->WinRead) + (OutputWindowPtr - IBState->WinSlide))));
         IBState->InfBlockMode = IBState->LastBlk ? BLK_DRY : BLK_TYPE;
         break;
    case BLK_TABLE:
         NEEDBITS(14)
         IBState->SubMode.Trees.TblLen = TempVal = (UInt)BitBuffer & 0x3fff;
#ifndef PKZIP_BUG_WORKAROUND
         if((TempVal & 0x1f) > 29 || ((TempVal >> 5) & 0x1f) > 29)
         {
           IBState->InfBlockMode = BLK_BAD;
           ReportErrorStr("too many length or distance symbols", Z_BAD_TOOMANY);
           RetCode = Z_DATA_ERROR;
           LEAVE
         }
#endif
         TempVal = 258 + (TempVal & 0x1f) + ((TempVal >> 5) & 0x1f);
         if((IBState->SubMode.Trees.BitLens = (UInt *) UnZipCalloc(TempVal, sizeof(UInt))) == Z_NULL)
         {
           ReportErrorStr("Buffer insufisant", Z_BAD_BUF_ERROR);
           RetCode = Z_MEM_ERROR;
           LEAVE
         }
         DUMPBITS(14)
         IBState->SubMode.Trees.Index = 0;
         Tracev((stderr, "inflate:       table sizes ok\n"));
         IBState->InfBlockMode = BLK_BTREE;
    case BLK_BTREE:
         while (IBState->SubMode.Trees.Index < 4 + (IBState->SubMode.Trees.TblLen >> 10))
         {
           NEEDBITS(3)
           IBState->SubMode.Trees.BitLens[ DefBorder [IBState->SubMode.Trees.Index++]] = (UInt)BitBuffer & 7;
           DUMPBITS(3)
         }
         while (IBState->SubMode.Trees.Index < 19){
           IBState->SubMode.Trees.BitLens[ DefBorder [IBState->SubMode.Trees.Index++]] = 0;
         }
         IBState->SubMode.Trees.BitDeep = 7;
         TempVal = InflateTreesBits(IBState->SubMode.Trees.BitLens, &IBState->SubMode.Trees.BitDeep,
                                   &IBState->SubMode.Trees.DecodeBit, IBState->hufts);
         if(TempVal   != Z_OK)
         {
           RetCode     = TempVal;
           if(RetCode == Z_DATA_ERROR)
           {
             UnZipFree(IBState->SubMode.Trees.BitLens);
             IBState->InfBlockMode = BLK_BAD;
           }
           LEAVE
         }
         IBState->SubMode.Trees.Index = 0;
         Tracev((stderr, "inflate:       bits tree ok\n"));
         IBState->InfBlockMode        = BLK_DTREE;
    case BLK_DTREE:
         while (TempVal = IBState->SubMode.Trees.TblLen,
                IBState->SubMode.Trees.Index < 258 + (TempVal & 0x1f) + ((TempVal >> 5) & 0x1f))
         {
           TInflate_Huft *TempHuft;
           UInt i, j, c;

           TempVal = IBState->SubMode.Trees.BitDeep;
           NEEDBITS(TempVal)
           TempHuft = IBState->SubMode.Trees.DecodeBit + ((UInt)BitBuffer & Inflate_Mask[TempVal]);
           TempVal = TempHuft->AsWord.What.Bits;
           c       = TempHuft->More.Base;
           if(c < 16)
           {
             DUMPBITS(TempVal)
             IBState->SubMode.Trees.BitLens[IBState->SubMode.Trees.Index++] = c;
           } else { // c == 16..18 
             i = c == 18 ? 7 : c - 14;
             j = c == 18 ? 11 : 3;
             NEEDBITS(TempVal + i)
             DUMPBITS(TempVal)
             j += (UInt)BitBuffer & Inflate_Mask[i];
             DUMPBITS(i)
             i = IBState->SubMode.Trees.Index;
             TempVal = IBState->SubMode.Trees.TblLen;
             if((i + j > 258 + (TempVal & 0x1f) + ((TempVal >> 5) & 0x1f))
             || (c == 16 && i < 1))
             {
               UnZipFree(IBState->SubMode.Trees.BitLens);
               IBState->InfBlockMode = BLK_BAD;
               ReportErrorStr("invalid bit length repeat", Z_BAD_BITLENREPEAT);
               RetCode = Z_DATA_ERROR;
               LEAVE
             }
             c = c == 16 ? IBState->SubMode.Trees.BitLens[i - 1] : 0;
             do {
               IBState->SubMode.Trees.BitLens[i++] = c;
             } while (--j);
             IBState->SubMode.Trees.Index = i;
           }
         }
         IBState->SubMode.Trees.DecodeBit = Z_NULL;
         {
           UInt                  LitBitDepth, DistBitDepth;
           TInflate_Huft        *LitTree,    *DistTree;

           LitBitDepth  = 9;         // must be <= 9 for lookahead assumptions 
           DistBitDepth = 6;         // must be <= 9 for lookahead assumptions 

           TempVal = IBState->SubMode.Trees.TblLen;
           TempVal = InflateTreesDynamic(257 + (TempVal & 0x1f), 1 + ((TempVal >> 5) & 0x1f),
                                         IBState->SubMode.Trees.BitLens, &LitBitDepth, &DistBitDepth,
                                        &LitTree, &DistTree, IBState->hufts);
           if(TempVal   != Z_OK)
           {
             if(TempVal == (UInt)Z_DATA_ERROR)
             {
               UnZipFree(IBState->SubMode.Trees.BitLens);
               IBState->InfBlockMode = BLK_BAD;
             }
             RetCode = TempVal;
             LEAVE
           }
           InflateCodesReset (LitBitDepth, DistBitDepth, LitTree, DistTree);
           IBState->SubMode.Decode.Codes = ICState;
         }
         UnZipFree(IBState->SubMode.Trees.BitLens);
         IBState->InfBlockMode = BLK_CODES;
    case BLK_CODES:
         UPDATE
         if((RetCode = InflateCodes(RetCode)) != Z_STREAM_END){
           return InflateFlush(RetCode);
         }
         RetCode = Z_OK;
         LOAD
         Tracev((stderr, "inflate:       codes end, %lu total out\n",
                 Zee->Total_out + (OutputWindowPtr >= IBState->WinRead ? OutputWindowPtr - IBState->WinRead :
                 (IBState->WinEnd - IBState->WinRead) + (OutputWindowPtr - IBState->WinSlide))));
         if(! IBState->LastBlk)
         {
           IBState->InfBlockMode = BLK_TYPE;
           break;
         }
         IBState->InfBlockMode = BLK_DRY;
    case BLK_DRY:
         FLUSH
         if(IBState->WinRead != IBState->WinWrite){
           LEAVE
         }
         IBState->InfBlockMode = BLK_DONE;
    case BLK_DONE:
         RetCode = Z_STREAM_END;
         LEAVE
    case BLK_BAD:
         RetCode = Z_DATA_ERROR;
         LEAVE
    default:
         RetCode = Z_STREAM_ERROR;
         LEAVE
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateBlocksFree(void)
{
  InflateBlocksReset();
  Tracev((stderr, "inflate:   blocks freed\n"));
  return Z_OK;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
