// inftrees.c -- generate Huffman trees for efficient decoding
#include "DevUnzip.h"
///////////////////////////////////////////////////////////////////////////////////////////////////
// Tables for deflate from PKZIP's appnote.txt. 
const UInt ProgMemmory _CopyLens [32]={
//local const UInt CopyLens [31] = { // Copy lengths for literal codes 257..285
		  3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
		  35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0};
		  // see note #13 above about 258
const UInt ProgMemmory _CopyLiteral [32]={
//local const UInt CopyLiteral [31] = { // Extra bits for literal codes 257..285
		  0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
		  3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, 112, 112}; // 112==invalid
const UInt ProgMemmory _CopyDist [30]={
//local const UInt CopyDist [30] = { // Copy offsets for distance codes 0..29
		  1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
		  257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
		  8193, 12289, 16385, 24577};
const UInt ProgMemmory _CopyExtraBits [30]={
//local const UInt CopyExtraBits [30] = { // Extra bits for distance codes
		  0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
		  7, 7, 8, 8, 9, 9, 10, 10, 11, 11,12, 12, 13, 13};

PUInt   CopyLens        = (PUInt)_CopyLens;
PUInt   CopyLiteral     = (PUInt)_CopyLiteral;
PUInt   CopyDist        = (PUInt)_CopyDist;
PUInt   CopyExtraBits   = (PUInt)_CopyExtraBits;

///////////////////////////////////////////////////////////////////////////////////////////////////
/*
   Huffman code decoding is performed using a multi-level table lookup.
   The fastest way to decode is to simply build a lookup table whose
   size is determined by the longest code.  However, the time it takes
   to build this table can also be a factor if the data being decoded
   is not very Long.  The most common codes are necessarily the
   shortest codes, so those codes dominate the decoding time, and hence
   the speed.  The idea is you can have a shorter table that decodes the
   shorter, more probable codes, and then point to subsidiary tables for
   the longer codes.  The time it costs to decode the longer codes is
   then traded against the time it takes to make longer tables.

   This results of this trade are in the variables lbits and dbits
   below.  lbits is the number of bits the first level table for literal/
   length codes can decode in one step, and dbits is the same thing for
   the distance codes.  Subsequent tables are also less than or equal to
   those sizes.  These values may be adjusted either when all of the
   codes are shorter than that, in which case the longest code length in
   bits is used, or when the shortest code is *longer* than the requested
   table size, in which case the length of the shortest code in bits is
   used.

   There are two different values for the two tables, since they code a
   different number of possibilities each.  The literal/length table
   codes 286 possible values, or in a flat code, a little over eight
   bits.  The distance table codes 30 possible values, or a little less
   than five bits, flat.  The optimum values for speed end up being
   about one bit more than those, so lbits is 8+1 and dbits is 5+1.
   The optimum values may differ though from machine to machine, and
   possibly even between compilers.  Your mileage may vary.
*/
///////////////////////////////////////////////////////////////////////////////////////////////////
// If BMAX needs to be larger than 16, then TblLevel and Offset_CodeStack[] should be ULong. 
#define BMAX 15         /* maximum bit length of any code */

///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
Int HuftBuild (
PUInt           CodeLen,                 /* code lengths in bits (all assumed <= BMAX) */
UInt            CodeCount,               /* number of codes (assumed <= 288) */
UInt            SimpleCodeCount,         /* number of simple-valued codes (0..SimpleCodeCount-1) */
const UInt     *BaseValTbl,              /* list of base values for non-simple codes */
const UInt     *ExtraBitTbl,             /* list of extra bits for non-simple codes */
TInflate_Huft **StartTbl,                /* result: starting table */
PUInt           LookupBits,              /* maximum lookup bits, returns actual */
TInflate_Huft  *TreesBuf,                /* space for trees */
PUInt           HuftsCount,              /* hufts used in space */
PUInt           StorageBuf )             /* working area: values in order of bit length */
///////////////////////////////////////////////////////////////////////////////////////////////////
/* Given a list of code lengths and a maximum table size, make a set of
   tables to decode that set of codes.  Return Z_OK on success, Z_BUF_ERROR
   if the given code set is incomplete (the tables are still built in this
   case), or Z_DATA_ERROR if the input is invalid.*/ 
///////////////////////////////////////////////////////////////////////////////////////////////////
{
UInt           BitLenCountTbl[BMAX+1];   /* bit length count table */
TInflate_Huft *TblStack[BMAX];           /* table stack */
UInt           Offset_CodeStack[BMAX+1]; /* bit offsets, then code stack */
UInt           LenCodeCount;             /* counter for codes of length CurCodeBitsCount */
UInt           RepeatDist;               /* CurCodeCount repeats in table every RepeatDist entries */
Int            MaxCodeLen;               /* maximum code length */
Int            TblLevel;                 /* table level */
register UInt  CurCodeCount;             /* counter, current code */
register UInt  Counter;                  /* counter */
register Int   CurCodeBitsCount;         /* number of bits in current code */
Int            BitsPerTbl;               /* bits per table (returned in LookupBits) */
UInt           Mask;                     /* (1 << BitsBefore) - 1, to avoid cc -O bug on HP */
register PUInt TblPtr;                   /* pointer into BitLenCountTbl[], CodeLen[], or StorageBuf[] */
TInflate_Huft *CurTbl;                   /* points to current table */
TInflate_Huft  TblEntry;                 /* table entry for structure assignment */
register Int   BitsBefore;               /* bits before this table == (BitsPerTbl * TblLevel) */
PUInt          Offset_CodeStackPtr;      /* pointer into Offset_CodeStack */
Int            DummyCodeCount;           /* number of dummy codes added */
UInt           TblEntryCount;            /* number of entries in current table */

//   StackPoint();
   
   /* Generate counts for each bit length */
   TblPtr = BitLenCountTbl;
   #define C0 *TblPtr++ = 0;
   #define C2 C0 C0 C0 C0
   #define C4 C2 C2 C2 C2
   C4                                    /* clear BitLenCountTbl[]--assume BMAX+1 is 16 */
   TblPtr       = CodeLen;  
   CurCodeCount = CodeCount;
   do {
      BitLenCountTbl[*TblPtr++]++;       /* assume all entries <= BMAX */
   } while (--CurCodeCount);

   if(BitLenCountTbl[0] == CodeCount){   /* null input--all zero length codes */
      *StartTbl   = (TInflate_Huft *)Z_NULL;
      *LookupBits = 0;
      return Z_OK;
   }
   /* Find minimum and maximum length, bound *LookupBits by those */
   BitsPerTbl = *LookupBits;
   for (Counter = 1; Counter <= BMAX; Counter++){
      if(BitLenCountTbl[Counter]){
         break;
      }
   }
   CurCodeBitsCount = Counter;          /* minimum code length */
   if((UInt)BitsPerTbl < Counter){
      BitsPerTbl = Counter;
   }
   for (CurCodeCount = BMAX; CurCodeCount; CurCodeCount--){
      if(BitLenCountTbl[CurCodeCount]){
         break;
      }
   }
   MaxCodeLen = CurCodeCount;           /* maximum code length */
   if((UInt)BitsPerTbl > CurCodeCount){
      BitsPerTbl = CurCodeCount;
   }
   *LookupBits = BitsPerTbl;

   /* Adjust last length count to fill out codes, if needed */
   for ((DummyCodeCount = 1 << Counter); Counter < CurCodeCount; (Counter++, DummyCodeCount <<= 1)){
      if((DummyCodeCount -= BitLenCountTbl[Counter]) < 0){
         return Z_DATA_ERROR;
      }
   }
   if((DummyCodeCount -= BitLenCountTbl[CurCodeCount]) < 0){
      return Z_DATA_ERROR;
   }
   BitLenCountTbl[CurCodeCount] += DummyCodeCount;

   /* Generate starting offsets into the value table for each length */
   Offset_CodeStack[1] = Counter = 0;
   TblPtr              = BitLenCountTbl + 1;  
   Offset_CodeStackPtr = Offset_CodeStack + 2;

   while (--CurCodeCount) {      /* note that CurCodeCount == MaxCodeLen from above */
      *Offset_CodeStackPtr++ = (Counter += *TblPtr++);
   }
   /* Make a table of values in order of bit lengths */
   TblPtr       = CodeLen;  
   CurCodeCount = 0;
   do {
      if((Counter = *TblPtr++) != 0){
         StorageBuf[Offset_CodeStack[Counter]++] = CurCodeCount;
      }
   } while (++CurCodeCount < CodeCount);

   CodeCount = Offset_CodeStack[MaxCodeLen];      /* set CodeCount to length of StorageBuf */

   /* Generate the Huffman codes and for each, make the table entries */
   Offset_CodeStack[0] = CurCodeCount = 0;        /* first Huffman code is zero */
   TblPtr              = StorageBuf;              /* grab values in bit order */
   TblLevel            = -1;                      /* no tables yet--level -1 */
   BitsBefore          = -BitsPerTbl;             /* bits decoded == (BitsPerTbl * TblLevel) */
   TblStack[0]         = (TInflate_Huft *)Z_NULL; /* just to keep compilers happy */
   CurTbl              = (TInflate_Huft *)Z_NULL; /* ditto */
   TblEntryCount       = 0;                       /* ditto */

   /* go through the bit lengths (CurCodeBitsCount already is bits in shortest code) */
   for (; CurCodeBitsCount <= MaxCodeLen; CurCodeBitsCount++){

      LenCodeCount = BitLenCountTbl[CurCodeBitsCount];
      while (LenCodeCount--){
         /* here CurCodeCount is the Huffman code of length CurCodeBitsCount bits for value *TblPtr */
         /* make tables up to required level */
         while (CurCodeBitsCount > BitsBefore + BitsPerTbl){
       
            TblLevel++;
            BitsBefore += BitsPerTbl;                    /* previous table always BitsPerTbl bits */

            /* compute minimum size table less than or equal to BitsPerTbl bits */
            TblEntryCount = MaxCodeLen - BitsBefore;
                                                          /* table size upper limit */
            TblEntryCount = TblEntryCount > (UInt)BitsPerTbl ? BitsPerTbl : TblEntryCount; 
                                                          /* try a CurCodeBitsCount-BitsBefore bit table */
            if((RepeatDist = 1 << (Counter = CurCodeBitsCount - BitsBefore)) > LenCodeCount + 1){ 
               /* too few codes for CurCodeBitsCount-BitsBefore bit table */
               RepeatDist         -= LenCodeCount + 1;    /* deduct codes from patterns left */
               Offset_CodeStackPtr = BitLenCountTbl + CurCodeBitsCount;
               if(Counter < TblEntryCount){
                  while (++Counter < TblEntryCount){     /* try smaller tables up to TblEntryCount bits */
                     if((RepeatDist <<= 1) <= (*++Offset_CodeStackPtr)){
                        break;                                   /* enough codes to use up Counter bits */
                     }
                     RepeatDist -= *Offset_CodeStackPtr;         /* else deduct codes from patterns */
                  }
               }
            }
            TblEntryCount = 1 << Counter;                        /* table entries for Counter-bit table */
            if(*HuftsCount + TblEntryCount > HOWMANY_BRANCH){    /* (note: doesn't matter for fixed) */
               return Z_DATA_ERROR;                              /* overflow of HOWMANY_BRANCH */
            }
            TblStack[TblLevel] = CurTbl = TreesBuf + *HuftsCount;
            *HuftsCount                += TblEntryCount;

            /* connect to last table, if there is one */
            if(TblLevel){
         
               Offset_CodeStack[TblLevel]    = CurCodeCount;      /* save pattern for backing up */
               TblEntry.AsWord.What.Bits     = (Byte)BitsPerTbl;  /* bits to dump before this table */
               TblEntry.AsWord.What.Ex_Op    = (Byte)Counter;     /* bits in this table */
               Counter = CurCodeCount       >> (BitsBefore - BitsPerTbl);
                                                                  /* offset to this table */
               TblEntry.More.Base            = (UInt)(CurTbl - TblStack[TblLevel-1] - Counter);
               TblStack[TblLevel-1][Counter] = TblEntry;          /* connect to last table */
            } else {
               *StartTbl                     = CurTbl;            /* first table is returned result */
            }              
         }
         /* set up table entry in TblEntry */
         TblEntry.AsWord.What.Bits = (Byte)(CurCodeBitsCount - BitsBefore);

         if(TblPtr >= StorageBuf + CodeCount){
            TblEntry.AsWord.What.Ex_Op = 128 + 64;                /* out of values--invalid code */

         } else if(*TblPtr < SimpleCodeCount){
                                                                  /* 256 is end-of-block */
            TblEntry.AsWord.What.Ex_Op = (Byte)(*TblPtr < 256 ? 0 : 32 + 64);     
            TblEntry.More.Base         = *TblPtr++;               /* simple code is just the value */
         } else {
       
            TblEntry.AsWord.What.Ex_Op = (Byte)(ExtraBitTbl[*TblPtr - SimpleCodeCount] + 16 + 64);/* non-simple--look up in lists */
            TblEntry.More.Base         = BaseValTbl[*TblPtr++ - SimpleCodeCount];
         }
         /* fill code-like entries with TblEntry */
         RepeatDist = 1 << (CurCodeBitsCount - BitsBefore);

         for ((Counter = CurCodeCount >> BitsBefore); Counter < TblEntryCount; Counter += RepeatDist){
            CurTbl[Counter] = TblEntry;
         }
         /* backwards increment the CurCodeBitsCount-bit code CurCodeCount */
         for ((Counter = 1 << (CurCodeBitsCount - 1)); (CurCodeCount & Counter); Counter >>= 1){
            CurCodeCount ^= Counter;
         }
         CurCodeCount ^= Counter;
         /* backup over finished tables */
         Mask = (1 << BitsBefore) - 1;        /* needed on HP, cc -O bug */

         while ((CurCodeCount & Mask) != Offset_CodeStack[TblLevel]){
            TblLevel   --;                    /* don't need to update CurTbl */
            BitsBefore -= BitsPerTbl;
            Mask        = (1 << BitsBefore) - 1;
         }
      }
   }
   /* Return Z_BUF_ERROR if we were given an incomplete table */
   return DummyCodeCount != 0 && MaxCodeLen != 1 ? Z_BUF_ERROR : Z_OK;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateTreesBits( 
PUInt           CodeLenTbl,    /* 19 code lengths */
PUInt           BitTreeDepth,  /* bits tree desired/actual depth */
TInflate_Huft **BitTreeRes,    /* bits tree result */
TInflate_Huft  *TreesBuf )     /* space for trees */
{
Int   RetCode;
UInt  HuftsCount = 0;          /* hufts used in space */

   ZMemClear((PByte)TreesBitsBuf, SIZEOF_TREEBITSBUFFER); 

   RetCode = HuftBuild(CodeLenTbl, 19, 19, (PUInt)Z_NULL, (PUInt)Z_NULL,
                       BitTreeRes, BitTreeDepth, TreesBuf, &HuftsCount, TreesBitsBuf);

   if(RetCode == Z_DATA_ERROR){

      ReportErrorStr("oversubscribed dynamic bit lengths tree", Z_BAD_OVERSUBDYNAMIC);

   } else if(RetCode == Z_BUF_ERROR || *BitTreeDepth == 0){

      ReportErrorStr("incomplete dynamic bit lengths tree", Z_BAD_DYNAMICLEN);
      RetCode = Z_DATA_ERROR;
   }
   return RetCode;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateTreesDynamic( 
UInt            NumOfLenCodes, /* number of literal/length codes */
UInt            DistCodeCount, /* number of distance codes */
PUInt           CodeLenCount,  /* that many (total) code lengths */
PUInt           LitBitDepth,   /* literal desired/actual bit depth */
PUInt           DistBitDepth,  /* distance desired/actual bit depth */
TInflate_Huft **LTreeRes,      /* literal/length tree result */
TInflate_Huft **DistTreeRes,   /* distance tree result */
TInflate_Huft  *TreesBuf )     /* space for trees */
{
Int   RetCode;
UInt  HuftsCount = 0;          /* hufts used in space */
                               /* build literal/length tree */
   ZMemClear((PByte)TreesDynBuf, SIZEOF_TREEDYNBUFFER);

   RetCode = HuftBuild(CodeLenCount, NumOfLenCodes, 257, CopyLens, CopyLiteral, LTreeRes, 
                       LitBitDepth, TreesBuf, &HuftsCount, TreesDynBuf);

   if(RetCode != Z_OK || *LitBitDepth == 0){
     switch(RetCode){
     case Z_DATA_ERROR:
          ReportErrorStr("oversubscribed literal/length tree", Z_BAD_OVERLITLEN);
          break;
     case Z_MEM_ERROR:
          ReportErrorStr("incomplete literal/length tree", Z_BAD_COMPLETELIT);
          RetCode = Z_DATA_ERROR;
          break;
     }
     return RetCode;
   }
   // BUILD DISTANCE TREE 
   RetCode = HuftBuild(CodeLenCount + NumOfLenCodes, DistCodeCount, 0, CopyDist, CopyExtraBits, 
                       DistTreeRes, DistBitDepth, TreesBuf, &HuftsCount, TreesDynBuf);

   if(RetCode != Z_OK || (*DistBitDepth == 0 && NumOfLenCodes > 257)){
      switch(RetCode){
      case Z_DATA_ERROR:
           ReportErrorStr("oversubscribed distance tree", Z_BAD_OVERDIST);
           break;
      case Z_BUF_ERROR:
           ReportErrorStr("incomplete distance tree", Z_BAD_COMPLETEDIST);
           RetCode = Z_DATA_ERROR;
           break;
      case Z_MEM_ERROR:
           ReportErrorStr("empty distance tree with lengths", Z_BAD_EMPTYDIST);
           RetCode = Z_DATA_ERROR;
           break;
      }
      return RetCode;
   }
   return Z_OK;
}
///////////////////////////////////////////////////////////////////////////////////////////////////



